/*
 * Copyright (C) 2001 USAGI/WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $USAGI: ksadbd.c,v 1.2 2001/07/19 07:00:55 mk Exp $
 */

#include <linux/config.h>
#include <linux/module.h>

#include <linux/types.h>
#include <linux/timer.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/smp_lock.h>
#include <linux/ipsec6.h>
#include <net/sadb.h>
#include <asm/system.h>
#include <asm/uaccess.h>
#include <asm/desc.h>

/* DEBUG *******************************************/
#ifdef CONFIG_IPV6_IPSEC_DEBUG
#define ipsec6_pr(fmt, arg...) printk(KERN_DEBUG fmt,##arg)
#else
#define ipsec6_pr(fmt,arg...) \
        do { } while (0)
#endif /* CONFIG_IPV6_IPSEC_DEBUG */

/**************************************************/

static DECLARE_WAIT_QUEUE_HEAD(ksadbd_waitqueue);

static int timeout = 30 ; /* 30 sec */
static int ksadbd_pid = 0;
static int exit_ksadbd;
static int ksadbd_running;

static void ksadbd_event_handler(void)
{
	ipsec6_pr("KSADBD: running\n");
	sadb_lifetime_scan();
}

static void ksadbd_mainloop(void)
{
	DECLARE_WAITQUEUE(wait, current);

	add_wait_queue(&ksadbd_waitqueue, &wait);
	set_current_state(TASK_INTERRUPTIBLE);

	for (;;) {
		int time = timeout * HZ;
#ifdef CONFIG_IPV6_IPSEC_DEBUG
		printk(KERN_DEBUG "ksadbd: time is %d\n", time);
#endif /* CONFIG_IPV6_IPSEC_DEBUG */
		schedule_timeout(time);
	
		if (exit_ksadbd)
			break;
		
		set_current_state(TASK_INTERRUPTIBLE);
		ksadbd_event_handler();
	}

	remove_wait_queue(&ksadbd_waitqueue, &wait);
}

		
static int ksadbd(void *unused)
{

	ksadbd_running = 1;

	daemonize();

	strcpy(current->comm, "ksadbd");
	sigfillset(&current->blocked);
	current->tty = NULL;    /* get rid of controlling tty */


	ksadbd_mainloop();

	ksadbd_running = 0;

	return 0;
}

int __init ksadbd_init(void)
{
	int pid;

	printk("starting ksadbd thread\n");
	pid = kernel_thread(ksadbd, NULL, CLONE_FS | CLONE_FILES | CLONE_SIGHAND | SIGCHLD);
	if (pid >= 0) {
		ksadbd_pid = pid;
		return 0;
	}

	printk(KERN_ERR "ERR: unable to start ksadbd\n");
	return -1;
}

void __exit ksadbd_exit(void)
{
	exit_ksadbd = 1;
	
	while (ksadbd_running)
		schedule();

	printk("exit ksadb thread\n");
}


#if 0
module_init(ksadbd_init);
module_exit(ksadbd_exit);

MODULE_AUTHOR("Mitsuru Kanda <mk@linux-ipv6.org>");
MODULE_DESCRIPTION("Kernel SADB thread");
MODULE_PARM(timeout, "i");
MODULE_PARM_DESC(timeout, "timeout parameter (unit is second)");
EXPORT_NO_SYMBOLS;
#endif
